import time
from threading import Thread
from blinkenbase.blinkenroom import BLINKENROOM_LOUNGE

class Animation(Thread):
    """
    An animation describes transition of pixels on various blinkenfoos. The
    blinkenfoo can not be changed after the animation has been initialized. It
    is possible to start an animation with a different blinkenroom everytime.
    """

    required_args = {}

    def __init__(self):
        Thread.__init__(self)
        self.name = None
        self.args = None
        self.blinkenroom = BLINKENROOM_LOUNGE
        self._frame = None
        self._fps = 0
        self._time_should = 0
        self._currently_adjusted_for = None
        self._running = False
        self._paused = False
        self._last_cycle = time.time()
        self.adjusted_fps_dict = {}

        self.set_fps(30)

    def set_args(self, p_args):
        """
        Passes the arguments to this animation before it is started
        """
        self.args = p_args

    #The fps of this animation are adjusted to the physikal width of the
    #blinkenfoos pixel. Default value ist 3.5 cm (pixel density of ws2812)

    def calc_devices_adjusted_fps(self, p_fps):
        """
        Pre-Calculates the adjusted fps of all devices in the specified
        blinkenroom.
        """
        for device in self.blinkenroom.device_list:
            adjusted_fps = device.calc_fps(p_fps)
            self.adjusted_fps_dict[device] = adjusted_fpss

    def adjust_fps_for_device(self, p_device):
        """
        Sets the fps of the whole animation to the adjusted fps of the given
        device.
        """
        adjusted_fps = self.adjusted_fps_dict[p_device]
        if not self._currently_adjusted_for == p_device:
            self.set_fps(adjusted_fps)
            self._currently_adjusted_for = p_device

    def adjust_fps_for_index(self, p_index):
        """
        Determine the device at the given index and sets the fps to the
        adjusted value.
        """
        device = self.blinkenroom.get_device_by_index(p_index)
        self.adjust_fps_for_device(device)

    def cycle(self):
        """
        Gets overriden by subclass
        """
        pass

    def set_fps(self, p_fps):
        """
        Sets the fps of the animation. Does not adjust fps.
        """
        self._fps = p_fps
        self._time_should = 1 / float(self._fps)

    def run(self):
        """
        Code stolen from olel
        """
        self._running = True
        while self._running:
            if self._paused:
                continue
            try:
                self.cycle()
            except:
                return
            if self._fps:
                time_spend = time.time() - self._last_cycle
                sleep_time = self._time_should - time_spend
                if sleep_time > 0:
                    time.sleep(sleep_time)
                self._last_cycle = time.time()

    def stop(self):
        """
        Sets the exit condition to true (running to false)
        """
        self._running = False

    def pause(self):
        self._paused = True

    def resume(self):
        self._paused = False
