from blinkenbase.animation import Animation
from blinkenbase.blinkenroom import Blinkenroom
import blinkenbase.blinkenroom
import blinkenbase.animation
import time
import os
from blinkenbase.blinkenroom import BLINKENROOM_LOUNGE
import _thread
import threading

class AnimationHandler(object):

    def __init__(self):
        self.running_animations = {}
        self.loaded_animation_instances = {}
        self.available_animations = []
        self.last_frame = None

        self._fetch_available_aninmations()

    def _fetch_available_aninmations(self):
        """
        Scans the animation directory and stores all animations
        """
        for root, dirs, files in os.walk("animations/"):
            for name in files:
                if name.endswith("py"):
                    name = name.split(".")[0]
                    self.available_animations.append(name)

    def get_available_animations(self):
        # TODO make fancier -> fstring
        return str(self.available_animations)

    #######################
    ###Animation control###
    ######################

    def stop_animation(self, p_name):
        if p_name in self.running_animations:
            animation = self.running_animations[p_name]
            print("Stopping %s" % p_name)
            animation.stop()
            del self.running_animations[p_name]

    def stop_all_animations(self, p_args=None):
        for running in self.running_animations.values():
            running.stop()
        self.running_animations = {}
        self.last_frame = BLINKENROOM_LOUNGE.get_last_frame()


    def pause_animation(self, p_animation_name):
        if p_animation_name in self.running_animations:
            animation = self.running_animations[p_animation_name]
            animation.pause()

    def pause_all_animations(self):
        for animation in self.running_animations.values():
            animation.pause()


    def resume_animation(self, p_animation_name):
        if p_animation_name in self.running_animations:
            animation = self.running_animations[p_animation_name]
            animation.resume()

    def resume_all_animations(self):
        for animation in self.running_animations.values():
            animation.resume()


    def _load_anmation(self, p_name):
        """
        Imports the given anmation and returns an instance of that class
        """
        exec("from animations.%s import %s" % (p_name, p_name))
        animation_class = eval(p_name)
        return animation_class()

    def start_animation(self, p_name, p_animation_args):
        """
        Loads an animation, creates and instance of it if none exists
        passes arguments to it and starts it
        """
        p_name = p_name.lower()

        animation = None
        if p_name not in self.loaded_animation_instances:
            try:
                animation = self._load_anmation(p_name)
            except ImportError:
                raise ValueError("Animation not found '%s'" % p_name)
        else:
            animation = self.loaded_animation_instances[p_name]

        #Loading args; Example pattern:
        #required_args = {
        #    "fps": [int, 20]
        #}
        required_args = animation.required_args
        for arg in required_args:
            required = required_args[arg]
            #Checking wether the specific arg was passed when calling the anim.
            if arg not in p_animation_args:
                #if not, loading default arg from class
                p_animation_args[arg] = required[1]
            else:
                #Casting passed arg
                casting_method = required[0]
                current_arg = p_animation_args[arg]
                p_animation_args[arg] = casting_method(current_arg)

        #Passing args to animation and doing init stuff
        animation.set_args(p_animation_args)
        animation.init_animation()

        #Stopping all animations that overlap
        to_stop = []
        for running in self.running_animations.values():
            if animation.blinkenroom.is_overlapping(running.blinkenroom):
                to_stop.append(running)
        for running in to_stop:
            #print("<<", type(running))
           # blinkenroom = running.blinkenroom
            #threading.Thread(target=self._thread_handler_darken, kwargs={"p_blinkenroom": blinkenroom, "p_sleep": 0.005}).start()
            self.stop_animation(running.name)

        #Finaly starting animation
        self.running_animations[p_name] = animation
        animation.start()
        

    #################
    ###Light-Level###
    #################

    def _thread_handler_darken(self, p_start=100, p_end=0, p_blinkenroom=BLINKENROOM_LOUNGE, p_sleep=0.05):
        """
        Method to be called by thread to darken all blinkendevices
        """
        print(p_start)
        print(p_end)
        f = p_start
        while f >= p_end:
            p_blinkenroom.set_brightness(f)
            time.sleep(p_sleep)
            f -= 1
        self.pause_all_animations()

    def _thread_handler_lighten(self, p_start=0, p_end=100, p_blinkenroom=BLINKENROOM_LOUNGE, p_sleep=0.05):
        """
        Method to be called by thread to light up all blinkendevices
        """
        f = p_start
        while f <= p_end:
            p_blinkenroom.set_brightness(f)
            time.sleep(p_sleep)
            f += 1

    def darken(self, p_command, p_args):
        start = 100
        end = 0

        if len(p_args) > 1:
            start = int(p_args[1])
        if len(p_args) > 2:
            end = int(p_args[2])
        #_thread.start_new_thread(self._thread_handler_darken, ())
        threading.Thread(target=self._thread_handler_darken).start()

    def lighten(self, p_command, p_args):
        #_thread.start_new_thread(self._thread_handler_lighten, ())
        self.resume_all_animations()
        threading.Thread(target=self._thread_handler_lighten).start()
