import socket
import sys

class BlinkenReceiver(object):
    """
    This class is the counterpart of blinkenbase.Blinkefoo.
    Blinkenfoo generates a frame and sends it as UDP, while as
    this class receives a udp package, turns it into a frame.
    Subclasses can implement a specific blinkefoo-device and display
    the frame in the desired fashion.
    """

    def __init__(self, p_name, p_hostname, p_port=2390):
        self.name = p_name
        self.hostname = p_hostname
        self.port = p_port

        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket.bind((self.hostname, self.port))

        self._running = False
    
    def start_listening(self, p_buffer_size=1024):
        self._running = True
        while self._running:
            data, address = self.socket.recvfrom(p_buffer_size)
            frame = self._decode_frame(data)
            print(frame)

    def stop_listening(self):
        self._running = False

    def _decode_frame(self, p_data):
        b = bytearray(p_data)
        length = len(b)
        start_index = 256 * b[0] + b[1]
        end_index = 256 * b[2] + b[3]
        rgb_list = []
        for i in range(4, length, 3):
            red = b[i]
            green = b[i+1]
            blue = b[i+2]
            rgb_list.append((red, green, blue))
        return ((start_index, end_index), rgb_list)


    def diplay_frame(self, p_frame):
        """
        Displays a given frame on the blinkenfoo device
        """
        pass
