from shell.mqtt_shell import MQTTShell
from shell.shellcore import Command
from blinkenbase.animation_handler import AnimationHandler

handler = AnimationHandler()
shell = MQTTShell("81!nk3n5h311", p_server="warpsrvint.warpzone")

def handler_start_animation(p_command, p_args):
    animation_name = p_args[1]
    animation_args_dict = {}
    for i in range(2, len(p_args), 2):
        key = str(p_args[i])
        value = p_args[i+1]
        animation_args_dict[key] = value

    try:
        handler.start_animation(animation_name, animation_args_dict)
    except ValueError as e:
        shell.print_error(str(e))

def handler_list_animations(p_command, p_args):
    shell.print_message(handler.get_available_animations())


def handler_pause(p_command, p_args):
    length = len(p_args)
    if length <= 1:
        handler.pause_all_animations()
    else:
        args = p_args[1:]
        for name in args:
            handler.pause_animation(name)

def handler_resume(p_command, p_args):
    length = len(p_args)
    if length <= 1:
        handler.resume_all_animations()
    else:
        args = p_args[1:]
        for name in args:
            handler.resume_animation(name)

command_pause = Command(
    p_name="pause",
    p_handler=handler_pause,
    p_usage="pause [<animation>]",
    p_help="Pauses a specific or all animations"
)

command_resume = Command(
    p_name="resume",
    p_handler=handler_resume,
    p_usage="resume [<animation>]",
    p_help="Resumes a specific or all animations"
)

command_play = Command(
    p_name="Play",
    p_handler=handler_start_animation,
    p_usage="play <animation> {<key> <value>}",
    p_help="Plays an animation. Eg. rainbow speed 2"
)

command_list = Command(
    p_name="List",
    p_handler=handler_list_animations,
    p_usage="list",
    p_help="Lists all available animations"
)

command_darken = Command(
    p_name="darken",
    p_handler=handler.darken,
    p_usage="darken",
    p_help="darkens all blinkenfoo to 0 and pauses all animations"
)

command_lighten = Command(
    p_name="lighten",
    p_handler=handler.lighten,
    p_usage="lighten",
    p_help="resumes all animations and lightens the room t0 100%"
)

shell.register_command(command_play)
shell.register_command(command_list)
shell.register_command(command_darken)
shell.register_command(command_lighten)
shell.register_command(command_pause)
shell.register_command(command_resume)

shell.start()
