from shell.shellcore import Shell
import paho.mqtt.client as mqtt

class MQTTShell(Shell):

    def __init__(self, p_name, p_server, p_port=1883, p_read_topic="blinkenfoo/read", p_write_topic="blinkenfoo/write"):
        super(MQTTShell, self).__init__(p_name)
        client = mqtt.Client()

        client.connect(p_server, p_port, 60)
        client.subscribe(p_read_topic)

        client.on_message = self._built_on_message()
        self.client = client
        self.write_topic = p_write_topic

    def _built_on_message(self):
        return lambda client, userdata, p_message: self.evaluate_input(p_message.payload.decode())

    def _write(self, p_message):
        self.client.publish(self.write_topic, p_message.encode())

    ##########################
    ### overriding methods ###
    ##########################

    def print_message(self, p_string):
        self._write(p_string)

    def print_notification(self, p_string):
        self.print_message(p_string)

    def print_error(self, p_string):
        self.print_message("[ Error ]" + p_string)

    def print_warning(self, p_string):
        self.print_message("[Warning]" + p_string)

    ###

    def start(self):
        super(MQTTShell, self).start(False)
        self.client.loop_forever()
